# Copyright 2016 Cumulus Networks, Inc. All rights reserved.

import falcon
import json

from restapi.utils.logs import get_logger
from restapi.apps.ml2.discovery import DiscoveryManager
from restapi.apps.ml2.l2 import L2Manager

_LOG = get_logger()

_NETWORKS = {
}

_PHYSICAL_INTERFACES = {
}


class BridgeCmd(object):

    def on_put(self, req, resp, network_id):
        body = req.stream.read()
        if not body:
            # XXX fixme
            resp.status = falcon.HTTP_200
            return
        json_data = json.loads(body.decode())
        _NETWORKS[network_id] = vlan_id = json_data['vlan']

        bridge_name = L2Manager.get_bridge_name(network_id)
        L2Manager.add_bridge(bridge_name, vlan_id, resp)

        # XXX fixme
        resp.status = falcon.HTTP_200

    def on_delete(self, _, resp, network_id):

        if network_id in _NETWORKS:
            bridge_name = L2Manager.get_bridge_name(network_id)
            vlan_id = _NETWORKS[network_id]

            L2Manager.delete_bridge(bridge_name, vlan_id, resp)

            del _NETWORKS[network_id]
        # XXX fixme
        resp.status = falcon.HTTP_200


class VxlanCmd(object):

    def on_put(self, _, resp, network_id, vni):
        bridge_name = L2Manager.get_bridge_name(network_id)
        vxlan_name = L2Manager.add_vxlan(vni, resp)

        L2Manager.add_interface(bridge_name, vxlan_name, resp)

        # XXX fixme
        resp.status = falcon.HTTP_200

    def on_delete(self, _, resp, network_id, vni):

        L2Manager.delete_vxlan(vni, resp)

        # XXX fixme
        resp.status = falcon.HTTP_200

class IntfCmd(object):

    def on_put(self, _, resp, network_id, host):
        physical_interface = DiscoveryManager.find_interface(host)
        if not physical_interface or physical_interface == 'eth0':
            # XXX fixme
            resp.status = falcon.HTTP_200
            return

        _PHYSICAL_INTERFACES[host] = physical_interface
        vlan_id = _NETWORKS[network_id]

        bridge_name = L2Manager.get_bridge_name(network_id)
        subinterface_name = L2Manager.add_vlan(physical_interface, vlan_id,
                                               resp)
        L2Manager.add_interface(bridge_name, subinterface_name, resp)

        # XXX fixme
        resp.status = falcon.HTTP_200

    def on_delete(self, _, resp, network_id, host):

        physical_interface = _PHYSICAL_INTERFACES.get(host)
        if not physical_interface:
            physical_interface = DiscoveryManager.find_interface(host)
            if not physical_interface or physical_interface == 'eth0':
                # XXX fixme
                resp.status = falcon.HTTP_200
                return

        vlan_id = _NETWORKS[network_id]
        bridge_name = L2Manager.get_bridge_name(network_id)

        subinterface_name = L2Manager.delete_interface(bridge_name,
                                                       physical_interface,
                                                       vlan_id, resp)

        L2Manager.delete_vlan(subinterface_name, resp)

        # XXX fixme
        resp.status = falcon.HTTP_200

